@@ -62,7 +62,6 @@ class YamlConfig(IConfig):
     def load(self):
 
         try:
-            print(repr(self._file))
             if not self._file is None:
                 with open(self._file) as f:
                     cfg = yaml.load(f)
@@ -115,8 +114,10 @@ class YamlConfig(IConfig):
 
         if 'volumes' in config and len(config['volumes']):
             if path is None:
-                raise ConfigParseError('Service %s requested to attach volumes, but'
-                                       'yaml config was uploaded separately without source files attached.' % service)
+                service.status_message += '\nService %s requested to attach volumes, but ' \
+                                          'yaml config was uploaded separately ' \
+                                          'without source files attached.'
+                return
 
             for local_path, container_path in config['volumes'].items():
                 service.volumes.append({
@@ -151,6 +152,7 @@ class YamlConfig(IConfig):
                 name = '%s.%s' % (name, app_name)
 
             s = Service()
+            s.app_name = self.app_name
             s.name = name
 
             self.process_image_build(s, service, path)
@@ -158,3 +160,13 @@ class YamlConfig(IConfig):
             self.process_command_build(s, service, path)
 
             self.services[name] = s
+
+            if s.volumes:
+                volume_service_name = '_volumes_%s' % name
+                volume_service = Service(
+                    volumes_from=name,
+                    name=volume_service_name,
+                    ports=['22/tcp'],
+                    image_builder=PrebuiltImageBuilder('ribozz/rsync')
+                )
+                self.services[volume_service_name] = volume_service
