@@ -1,9 +1,12 @@
+
 import json
 import logging
 import sys
 import inject
 from mfcloud.util import txtimeout
+import os
 from twisted.internet import reactor
+from twisted.internet.defer import inlineCallbacks
 import txredisapi
 from txzmq import ZmqFactory, ZmqEndpoint, ZmqSubConnection
 
@@ -50,57 +53,91 @@ backend {{ app.name }}_cluster
 from jinja2 import Environment as JinjaEnv, StrictUndefined, Template
 
 
-class HaproxyConfig(object):
+class DnsConfig(object):
 
     redis = inject.attr(txredisapi.Connection)
 
-    def __init__(self, path, template=None, internal_suffix='local'):
+    def dump(self, apps_list):
+        apps = {}
+
+        for app in apps_list:
+            if not app['running']:
+                continue
+
+            for service in app['services']:
+                apps[service['fullname']] = service['ip']
+
+            if app['web_service']:
+                apps[app['fullname']] = app['web_ip']
+
+            if app['public_url']:
+                apps[app['public_url']] = app['web_ip']
+
+        logging.info('Installing new app list: %s' % str(apps))
+
+        if len(apps) > 1:
+            return self.redis.hmset('domain', apps)
+        elif len(apps) == 1:
+            return self.redis.hset('domain', apps.keys()[0], apps.values()[0])
+
+
+class HaproxyConfig(object):
+
+    def __init__(self, path, template=None, internal_suffix='mfcloud.lh'):
         self.template = template
         self.path = path
-        self.internal_suffix = internal_suffix
 
-    def dump(self, containers):
+    def dump(self, apps_list):
 
         template = self.template
 
         if not template:
             template = Template(HAPROXY_TPL)
 
-        hosts = {}
+        proxy_apps = []
 
-        # apps = []
-        for container in containers:
-            name_ = container['Name']
-            if name_[0] == '/':
-                name_ = name_[1:]
+        for app in apps_list:
+            if not app['running']:
+                continue
 
-            name_ = '.'.join([name_, self.internal_suffix])
+            domains = [app['fullname']]
 
-            hosts[name_] = container['NetworkSettings']['IPAddress']
-
-        return self.redis.hmset('domain', hosts)
+            if app['public_url']:
+                domains.append(app['public_url'])
 
 
+            proxy_apps.append({
+                'name': app['fullname'],
+                'domains': domains,
+                'backends': [{
+                     'name': 'backend_%s' % app['fullname'],
+                     'ip': app['web_ip'],
+                     'port': 80
+                 }]
+            })
 
-            # apps.append({
-            #     'name': re.sub('[^a-z0-9_]', '_', conf['domain']),
-            #     'domains': [conf['domain']],
-            #     'backends': [{
-            #                      'name': 'backend_%s' % i,
-            #                      'ip': x[0],
-            #                      'port': x[1]
-            #                  } for i, x in enumerate(conf['backends'])]
-            # })
+        with open('/etc/haproxy/haproxy.cfg', 'w') as f:
+            f.write(template.render({'apps': proxy_apps}))
+
+        os.system('service haproxy reload')
+
+
+def listen_events(zf2, endpoint, need_haproxy=False):
+    proxy_config = HaproxyConfig(path='/etc/haproxy/haproxy.cfg')
+    dns_config = DnsConfig()
+
+    def on_message(message, tag):
+        if tag == 'event-containers-updated':
+            data = json.loads(message)
+            dns_config.dump(data['apps'])
+
+            if need_haproxy:
+                proxy_config.dump(data['apps'])
+
+    subscribe_con = ZmqSubConnection(zf2, ZmqEndpoint('connect', endpoint))
+    subscribe_con.subscribe("")
+    subscribe_con.gotMessage = on_message
 
-        #
-        #
-        #
-        # template = Template(HAPROXY_TPL)
-        #
-        # with open(path, 'w') as f:
-        #     f.write(template.render({'apps': apps}))
-        #
-        # os.system('service haproxy reload')
 
 def entry_point():
 
@@ -119,22 +156,11 @@ def entry_point():
     parser.add_argument('--endpoint', type=str, default=['tcp://127.0.0.1:5555'], nargs='+', help='ip address')
     args = parser.parse_args()
 
+
+
     def run_server(redis):
-        config = HaproxyConfig(path='/etc/haproxy/haproxy.cfg')
-
-        def on_message(message, tag):
-            if tag == 'event-containers-updated':
-                config.dump(json.loads(message)['list'])
-
-
-        print args.endpoint
-
         zf2 = ZmqFactory()
-
-        for endpoint in args.endpoint:
-            subscribe_con = ZmqSubConnection(zf2, ZmqEndpoint('connect', endpoint))
-            subscribe_con.subscribe("")
-            subscribe_con.gotMessage = on_message
+        listen_events(zf2, args.endpoint)
 
         def my_config(binder):
             binder.bind(txredisapi.Connection, redis)
@@ -148,6 +174,8 @@ def entry_point():
 
     txtimeout(txredisapi.Connection(dbid=1), 3, timeout).addCallback(run_server)
 
+
+
     reactor.run()
 
 if __name__ == '__main__':
