@@ -1,12 +1,11 @@
 import json
 import logging
 import sys
-from mfcloud.config import ConfigParseError
 import os
 import pprintpp
-from prettytable import PrettyTable
+from prettytable import PrettyTable, ALL
 from texttable import Texttable
-from twisted.internet import defer, reactor
+from twisted.internet import reactor
 from twisted.internet.error import ConnectionRefusedError
 from twisted.web.xmlrpc import Proxy
 from txzmq import ZmqFactory, ZmqEndpoint, ZmqSubConnection
@@ -16,13 +15,16 @@ logger = logging.getLogger('mfcloud.client')
 
 class ApiRpcClient(object):
 
-    def __init__(self):
+    def __init__(self, host='0.0.0.0'):
         super(ApiRpcClient, self).__init__()
 
+        self.host = host
+
         self.init_zmq()
 
         self.ticket = {}
-        self.proxy = Proxy('http://127.0.0.1:7080')
+
+        self.proxy = Proxy('http://%s:7080' % host)
 
         self.reactor = reactor
 
@@ -31,7 +33,7 @@ class ApiRpcClient(object):
 
     def init_zmq(self):
         zf2 = ZmqFactory()
-        e2 = ZmqEndpoint('connect', 'tcp://127.0.0.1:5555')
+        e2 = ZmqEndpoint('connect', 'tcp://%s:5555' % self.host)
         s2 = ZmqSubConnection(zf2, e2)
         s2.subscribe("")
         s2.gotMessage = self._on_message
@@ -95,7 +97,6 @@ class ApiRpcClient(object):
 
                 elif 'status' in data:
                     sys.stdout.write('\n%s' % (data['status']))
-
                 else:
                     if isinstance(data, basestring):
                         sys.stdout.write(data)
@@ -110,31 +111,86 @@ class ApiRpcClient(object):
         def on_result(data):
             print 'result: %s' % pprintpp.pformat(data)
 
-        self._remote_exec('init', on_result, name, os.path.realpath(path))
+        self._remote_exec('init', self.on_print_list_result, name, os.path.realpath(path))
+
+
+    def on_print_list_result(self, data):
+
+        x = PrettyTable(["Application name", "Web", "status", "services"], hrules=ALL)
+        for app in data:
+
+            volume_services = {}
+
+            for service in app['services']:
+                if service['name'].startswith('_volumes_') and service['running']:
+                    name = service['name']
+                    if name.endswith(app['name']):
+                        name = name[0:-len(app['name']) - 1]
+                    name = name[9:]
+                    print service
+                    volume_services[name] = '%s' % (
+                        #service['ports']['22/tcp'][0]['HostIp'],
+                        service['ports']['22/tcp'][0]['HostPort'],
+                    )
+
+            services = []
+            for service in app['services']:
+                if service['name'].startswith('_volumes_'):
+                    continue
+
+                name = service['name']
+                if name.endswith(app['name']):
+                    name = name[0:-len(app['name']) - 1]
+
+                if service['created']:
+                    service_status = 'ON' if service['running'] else 'OFF'
+                else:
+                    service_status = 'NOT CREATED'
+
+                if service['is_web']:
+                    mark = '*'
+                else:
+                    mark = ''
+
+                data = '%s%s (%s)' % (name, mark, service_status)
+
+                if service['ip']:
+                    data += ' ip: %s' % service['ip']
+
+                if name in volume_services:
+                    data += ' vol: %s' % volume_services[name]
+                    data += ' (%s)' % ', '.join(service['volumes'])
+
+                services.append(data)
+
+            if app['running']:
+                app_status = app['status']
+                services_list = '\n'.join(services)
+            elif app['status'] == 'error':
+                app_status = 'ERROR'
+                services_list = app['message']
+            else:
+                app_status = ''
+                services_list = '\n'.join(services)
+
+            if 'web_service' in app and app['web_service']:
+                web_service_ = app['web_service']
+                if web_service_.endswith(app['name']):
+                     web_service_ = web_service_[0:-len(app['name']) - 1]
+                web = '%s -> [%s]' % (app['fullname'], web_service_)
+            else:
+                web = ''
+
+            if 'public_url' in app and app['public_url']:
+                web += '\n' + app['public_url']
+
+            x.add_row([app['name'], web, app_status, services_list])
+
+        print x
 
     def list(self, **kwargs):
+        self._remote_exec('list', self.on_print_list_result)
 
-        def on_result(data):
-
-            x = PrettyTable(["Application name", "Application path"])
-            for row in data:
-                x.add_row(row)
-            print x
-
-        self._remote_exec('list', on_result)
-
-    def status(self, name, **kwargs):
-
-        def on_result(data):
-
-            print 'Services:'
-
-            x = PrettyTable(["Service name", "is created", "is running"])
-            for row in data:
-                x.add_row(row)
-            print x
-
-        self._remote_exec('status', on_result, name)
 
     def inspect(self, name, service, **kwargs):
 
@@ -163,29 +219,35 @@ class ApiRpcClient(object):
         def on_result(data):
             print 'result: %s' % pprintpp.pformat(data)
 
-        self._remote_exec('remove', on_result, name)
+        self._remote_exec('remove', self.on_print_list_result, name)
+
+    def destroy(self, name, **kwargs):
+
+        def on_result(data):
+            print 'result: %s' % pprintpp.pformat(data)
+
+        self._remote_exec('destroy', self.on_print_list_result, name)
 
     def start(self, name, **kwargs):
 
         def on_result(data):
             print 'result: %s' % pprintpp.pformat(data)
 
-        self._remote_exec('start', on_result, name)
+        self._remote_exec('start', self.on_print_list_result, name)
+
+    def push(self, source, destination, **kwargs):
+        
+        def on_result(data):
+            print 'result: %s' % pprintpp.pformat(data)
+
+            self._remote_exec('push', self.on_print_list_result, source, destination)
 
     def stop(self, name, **kwargs):
 
         def on_result(data):
             print 'result: %s' % pprintpp.pformat(data)
 
-        self._remote_exec('stop', on_result, name)
-
-
-    def list_deployments(self, **kwargs):
-
-        def on_result(data):
-            print 'result: %s' % pprintpp.pformat(data)
-
-        self._remote_exec('deployments', on_result)
+        self._remote_exec('stop', self.on_print_list_result, name)
 
 
 def populate_client_parser(subparsers):
@@ -203,20 +265,22 @@ def populate_client_parser(subparsers):
     cmd.add_argument('name', help='App name')
     cmd.set_defaults(func='remove')
 
-    cmd = subparsers.add_parser('status', help='Remove application')
-    cmd.add_argument('name', help='App name')
-    cmd.set_defaults(func='status')
-
     cmd = subparsers.add_parser('start', help='Start application')
     cmd.add_argument('name', help='App name')
     cmd.set_defaults(func='start')
 
+    cmd = subparsers.add_parser('push', help='Push volume')
+    cmd.add_argument('source', help='Push source')
+    cmd.add_argument('destination', help='Push destination')
+    cmd.set_defaults(func='push')
+
     cmd = subparsers.add_parser('stop', help='Stop application')
     cmd.add_argument('name', help='App name')
     cmd.set_defaults(func='stop')
 
-    cmd = subparsers.add_parser('list_deployments', help='Stop application')
-    cmd.set_defaults(func='list_deployments')
+    cmd = subparsers.add_parser('destroy', help='Destroy application containers')
+    cmd.add_argument('name', help='App name')
+    cmd.set_defaults(func='destroy')
 
     cmd = subparsers.add_parser('inspect', help='Inspect application service')
     cmd.add_argument('name', help='App name')
@@ -232,35 +296,6 @@ def populate_client_parser(subparsers):
     # fig_cmd.set_defaults(func=func=fig_main)
 
 
-    # cmd = subparsers.add_parser('start', help='Run services as daemons')
-    # cmd.add_argument('services', help='Service names', nargs='*')
-    # cmd.add_argument('--no-logs', dest='logs', action='store_false', default=True, help='No logs')
-    # cmd.add_argument('--rebuild', action='store_true', default=False, help='Rebuild container')
-    # cmd.set_defaults(func='start')
-
-    # cmd = subparsers.add_parser('run', help='Run command on a service')
-    # cmd.add_argument('service', help='Service name')
-    # cmd.add_argument('command', help='Command to run')
-    # cmd.add_argument('--no-tty', dest='disable_tty', action='store_true', default=False, help='No tty')
-    # cmd.set_defaults(func='run')
-    #
-    # cmd = subparsers.add_parser('stop', help='Stop services')
-    # cmd.add_argument('services', help='Service names', nargs='*')
-    # cmd.set_defaults(func='stop')
-    #
-    # cmd = subparsers.add_parser('destroy', help='Destory services and containers')
-    # cmd.add_argument('services', help='Service names', nargs='*')
-    # cmd.set_defaults(func='destroy')
-    #
-    # cmd = subparsers.add_parser('rebuild', help='Destory services and containers')
-    # cmd.add_argument('services', help='Service names', nargs='*')
-    # cmd.set_defaults(func='rebuild')
-    #
-    # cmd = subparsers.add_parser('logs', help='Fetch logs from containers')
-    # cmd.add_argument('services', help='Service names', nargs='*')
-    # cmd.set_defaults(func='logs')
-
-
     # 'PS1=(.env)\[\e]0;\u@\h: \w\a\]${debian_chroot:+($debian_chroot)}\u@\h:\w\$'
 
 
@@ -281,4 +316,103 @@ def populate_client_parser(subparsers):
     # cmd.set_defaults(func='status')
 
 
+import argparse
+from cmd import Cmd
 
+from mfcloud import metadata
+
+
+log = logging.getLogger(__name__)
+
+def format_epilog():
+    """Program entry point.
+
+    :param argv: command-line arguments
+    :type argv: :class:`list`
+    """
+    author_strings = []
+    for name, email in zip(metadata.authors, metadata.emails):
+        author_strings.append('Author: {0} <{1}>'.format(name, email))
+    epilog = '''
+{project} {version}
+
+{authors}
+URL: <{url}>
+'''.format(
+        project=metadata.project,
+        version=metadata.version,
+        authors='\n'.join(author_strings),
+        url=metadata.url)
+    return epilog
+
+
+def main(argv):
+
+    console_handler = logging.StreamHandler(stream=sys.stderr)
+    console_handler.setFormatter(logging.Formatter())
+    console_handler.setLevel(logging.DEBUG)
+
+    root_logger = logging.getLogger()
+    root_logger.addHandler(console_handler)
+    root_logger.setLevel(logging.INFO)
+    root_logger.debug('Logger initialized')
+
+
+    logging.getLogger("requests").propagate = False
+
+    arg_parser = argparse.ArgumentParser(
+        prog=argv[0],
+
+        formatter_class=argparse.RawDescriptionHelpFormatter,
+        description=metadata.description,
+        epilog=format_epilog(),
+        add_help=False
+    )
+
+    arg_parser.add_argument('-e', '--env', help='Environment to use', default='dev')
+    arg_parser.add_argument('-h', '--host', help='Host to use', default='127.0.0.1')
+
+    arg_parser.add_argument(
+        '-V', '--version',
+        action='version',
+        version='{0} {1}'.format(metadata.project, metadata.version))
+
+    subparsers = arg_parser.add_subparsers()
+
+    populate_client_parser(subparsers)
+
+
+
+
+    args = arg_parser.parse_args()
+
+    args.argv0 = argv[0]
+
+    client = ApiRpcClient(host=args.host)
+
+    if isinstance(args.func, str):
+        getattr(client, args.func)(**vars(args))
+    else:
+        args.func(**vars(args))
+
+
+
+
+# intro = '''
+#            __      _                 _
+# _ __ ___  / _| ___| | ___  _   _  __| |
+#| '_ ` _ \| |_ / __| |/ _ \| | | |/ _` |
+#| | | | | |  _| (__| | (_) | |_| | (_| |
+#|_| |_| |_|_|  \___|_|\___/ \__,_|\__,_|
+#
+#Cloud that loves your data.
+#
+#'''
+
+def entry_point():
+    """Zero-argument entry point for use with setuptools/distribute."""
+    raise SystemExit(main(sys.argv))
+
+
+if __name__ == '__main__':
+    entry_point()
